#!/bin/bash

get_ipv6_prefix()
{
    ipv6_address="$1"

    prefix_length=${ipv6_address#*/}

    ipv6=${ipv6_address%/*}

    IFS=':' read -ra blocks <<< "$ipv6"

    ipv6_prefix=""

    full_blocks=$(($prefix_length / 16))
    partial_block_bits=$(($prefix_length % 16))

    for ((i=0; i<full_blocks; i++)); do
        ipv6_prefix+=${blocks[i]}:
    done

    if [[ $partial_block_bits -ne 0 && $full_blocks -lt 8 ]]; then
        partial_block_value=$(printf '%x' "$((0x${blocks[full_blocks]} & (0xffff << (16 - partial_block_bits))))")
        ipv6_prefix+=$partial_block_value
    fi

    echo "$ipv6_prefix:/$prefix_length"
}

if ! command -v jq 2>&1 >/dev/null
then
        echo No jq. exiting...
        exit 1
fi


NICS=$(jq -M '.nics[]' /etc/pear/pear_monitor/config.json | tr -d '"')

for N in $NICS
do
    IP6=$(ip -6 -j address show primary | jq -M '.[] | select(.ifname=="'"$N"'").addr_info[] | select(.scope=="global") | .local as $local | (.prefixlen | tostring) as $prefixlen | {$local, $prefixlen}')

    if [ -z "$IP6" ]
    then
        continue
    fi

    SRC_IP6=$(echo "$IP6" | jq -M -r '.local+"/"+.prefixlen' )
    SRC_IP6=$(get_ipv6_prefix "$SRC_IP6")

    ip -6 rule flush lookup $N
    ip -6 rule add from $SRC_IP6 lookup $N priority 1024
    ip -6 rule add to $SRC_IP6 lookup $N priority 1024

    logger -p local0.notice -t ${0##*/}[$$] "Added rules for $N address $SRC_IP6"
done

